@@ -42,7 +42,7 @@ pub trait VariantMatcher: Debug {
   ) -> Option<(usize, HashMap<DocPath, Generator>)>;
 
   /// Clones this matcher and returns it in a box
-  fn boxed(&self) -> Box<dyn VariantMatcher>;
+  fn boxed(&self) -> Box<dyn VariantMatcher + Send + Sync>;
 }
 
 #[derive(Clone, Debug)]
@@ -57,7 +57,7 @@ impl VariantMatcher for NoopVariantMatcher {
     None
   }
 
-  fn boxed(&self) -> Box<dyn VariantMatcher> {
+  fn boxed(&self) -> Box<dyn VariantMatcher + Send + Sync> {
     Box::new(self.clone())
   }
 }
@@ -113,7 +113,7 @@ impl FromStr for UuidFormat {
 }
 
 /// Trait to represent a generator
-#[derive(Serialize, Deserialize, Debug, Clone, Eq)]
+#[derive(Debug, Clone, Eq)]
 pub enum Generator {
   /// Generates a random integer between the min and max values
   RandomInt(i32, i32),
@@ -287,6 +287,15 @@ impl Generator {
       }
     }
   }
+
+  /// Create a generator from a type and a map of attributes
+  pub fn create(generator_type: &str, attributes: &Value) -> anyhow::Result<Generator> {
+    match attributes {
+      Value::Object(o) => Generator::from_map(generator_type, o)
+        .ok_or_else(|| anyhow!("Could not create a generator from '{}' and {}", generator_type, attributes)),
+      _ => Err(anyhow!("'{}' is not a valid generator type", generator_type))
+    }
+  }
 }
 
 impl Hash for Generator {
@@ -581,7 +590,7 @@ pub enum GeneratorTestMode {
 
 
 /// Category that the generator is applied to
-#[derive(Serialize, Deserialize, PartialEq, Debug, Clone, Eq, Hash)]
+#[derive(PartialEq, Debug, Clone, Eq, Hash)]
 pub enum GeneratorCategory {
   /// Request Method
   METHOD,
@@ -651,7 +660,12 @@ impl Into<Category> for GeneratorCategory {
 pub trait GenerateValue<T> {
   /// Generates a new value based on the source value. An error will be returned if the value can not
   /// be generated.
-  fn generate_value(&self, value: &T, context: &HashMap<&str, Value>, matcher: &Box<dyn VariantMatcher>) -> anyhow::Result<T>;
+  fn generate_value(
+    &self,
+    value: &T,
+    context: &HashMap<&str, Value>,
+    matcher: &Box<dyn VariantMatcher + Send + Sync>
+  ) -> anyhow::Result<T>;
 }
 
 /// Trait to define a handler for applying generators to data of a particular content type.
@@ -662,7 +676,7 @@ pub trait ContentTypeHandler<T> {
     generators: &HashMap<DocPath, Generator>,
     mode: &GeneratorTestMode,
     context: &HashMap<&str, Value>,
-    matcher: &Box<dyn VariantMatcher>
+    matcher: &Box<dyn VariantMatcher + Send + Sync>
   ) -> Result<OptionalBody, String>;
   /// Applies the generator to the key in the body.
   fn apply_key(
@@ -670,13 +684,12 @@ pub trait ContentTypeHandler<T> {
     key: &DocPath,
     generator: &dyn GenerateValue<T>,
     context: &HashMap<&str, Value>,
-    matcher: &Box<dyn VariantMatcher>
+    matcher: &Box<dyn VariantMatcher + Send + Sync>
   );
 }
 
 /// Data structure for representing a collection of generators
-#[derive(Serialize, Deserialize, Debug, Clone, Eq)]
-#[serde(transparent)]
+#[derive(Debug, Clone, Eq)]
 pub struct Generators {
   /// Map of generator categories to maps of generators
   pub categories: HashMap<GeneratorCategory, HashMap<DocPath, Generator>>
@@ -785,6 +798,16 @@ impl Generators {
     let category_map = self.categories.entry(category.clone()).or_insert(HashMap::new());
     category_map.insert(subcategory, generator.clone());
   }
+
+  /// Add all the generators
+  pub fn add_generators(&mut self, generators: Generators) {
+    for (key, values) in &generators.categories {
+      let category_map = self.categories.entry(key.clone()).or_insert(HashMap::new());
+      for (path, gen) in values {
+        category_map.insert(path.clone(), gen.clone());
+      }
+    }
+  }
 }
 
 impl Hash for Generators {
@@ -949,7 +972,12 @@ pub fn generate_hexadecimal(digits: usize) -> String {
 }
 
 impl GenerateValue<u16> for Generator {
-  fn generate_value(&self, value: &u16, context: &HashMap<&str, Value>, _matcher: &Box<dyn VariantMatcher>) -> anyhow::Result<u16> {
+  fn generate_value(
+    &self,
+    value: &u16,
+    context: &HashMap<&str, Value>,
+    _matcher: &Box<dyn VariantMatcher + Send + Sync>
+  ) -> anyhow::Result<u16> {
     match self {
       &Generator::RandomInt(min, max) => Ok(rand::thread_rng().gen_range(min as u16..(max as u16).saturating_add(1))),
       &Generator::ProviderStateGenerator(ref exp, ref dt) =>
@@ -988,7 +1016,12 @@ fn replace_with_regex(example: &String, url: String, re: Regex) -> String {
 }
 
 impl GenerateValue<String> for Generator {
-  fn generate_value(&self, _: &String, context: &HashMap<&str, Value>, _matcher: &Box<dyn VariantMatcher>) -> anyhow::Result<String> {
+  fn generate_value(
+    &self,
+    _: &String,
+    context: &HashMap<&str, Value>,
+    _matcher: &Box<dyn VariantMatcher + Send + Sync>
+  ) -> anyhow::Result<String> {
     let mut rnd = rand::thread_rng();
     let result = match self {
       Generator::RandomInt(min, max) => Ok(format!("{}", rnd.gen_range(*min..max.saturating_add(1)))),
@@ -1077,13 +1110,23 @@ impl GenerateValue<String> for Generator {
 }
 
 impl GenerateValue<Vec<String>> for Generator {
-  fn generate_value(&self, vals: &Vec<String>, context: &HashMap<&str, Value>, matcher: &Box<dyn VariantMatcher>) -> anyhow::Result<Vec<String>> {
+  fn generate_value(
+    &self,
+    vals: &Vec<String>,
+    context: &HashMap<&str, Value>,
+    matcher: &Box<dyn VariantMatcher + Send + Sync>
+  ) -> anyhow::Result<Vec<String>> {
     self.generate_value(&vals.first().cloned().unwrap_or_default(), context, matcher).map(|v| vec![v])
   }
 }
 
 impl GenerateValue<Value> for Generator {
-  fn generate_value(&self, value: &Value, context: &HashMap<&str, Value>, matcher: &Box<dyn VariantMatcher>) -> anyhow::Result<Value> {
+  fn generate_value(
+    &self, value:
+    &Value,
+    context: &HashMap<&str, Value>,
+    matcher: &Box<dyn VariantMatcher + Send + Sync>
+  ) -> anyhow::Result<Value> {
     debug!("Generating value from {:?} with context {:?}", self, context);
     let result = match self {
       Generator::RandomInt(min, max) => {
@@ -1294,7 +1337,7 @@ impl ContentTypeHandler<Value> for JsonHandler {
     generators: &HashMap<DocPath, Generator>,
     mode: &GeneratorTestMode,
     context: &HashMap<&str, Value>,
-    matcher: &Box<dyn VariantMatcher>
+    matcher: &Box<dyn VariantMatcher + Send + Sync>
   ) -> Result<OptionalBody, String> {
     for (key, generator) in generators {
       if generator.corresponds_to_mode(mode) {
@@ -1302,7 +1345,7 @@ impl ContentTypeHandler<Value> for JsonHandler {
         self.apply_key(key, generator, context, matcher);
       }
     };
-    Ok(OptionalBody::Present(self.value.to_string().into(), Some("application/json".into())))
+    Ok(OptionalBody::Present(self.value.to_string().into(), Some("application/json".into()), None))
   }
 
   fn apply_key(
@@ -1310,7 +1353,7 @@ impl ContentTypeHandler<Value> for JsonHandler {
     key: &DocPath,
     generator: &dyn GenerateValue<Value>,
     context: &HashMap<&str, Value>,
-    matcher: &Box<dyn VariantMatcher>,
+    matcher: &Box<dyn VariantMatcher + Send + Sync>,
   ) {
     let path_exp = key;
     let mut tree = Arena::new();
@@ -1353,7 +1396,7 @@ mod tests {
   use expectest::expect;
   use expectest::prelude::*;
   use hamcrest2::*;
-  use test_env_log::test;
+  use test_log::test;
 
   use crate::generators::Generator::{RandomDecimal, RandomInt, Regex};
 
