@@ -1,24 +1,26 @@
 //! Synchronous HTTP Request/Response Pact
 
 use std::cmp::Ordering;
-use std::collections::BTreeMap;
+use std::collections::{BTreeMap, HashMap};
 use std::path::Path;
 use std::sync::{Arc, Mutex};
 
 use anyhow::{anyhow, Context};
-use itertools::{iproduct, Itertools};
 use itertools::EitherOrBoth::{Both, Left, Right};
+use itertools::Itertools;
 use log::warn;
 use maplit::{btreemap, hashset};
-use serde_json::{json, Value};
+use serde_json::{json, Value, Map};
 
 use crate::{Consumer, PactSpecification, Provider};
 #[cfg(not(target_family = "wasm"))] use crate::file_utils::with_read_lock;
 #[cfg(not(target_family = "wasm"))] use crate::http_utils::{self, HttpAuth};
 use crate::interaction::{Interaction, PactConflict, parse_interactions};
+use crate::iterator_utils::CartesianProductIterator;
 use crate::message_pact::MessagePact;
 use crate::pact::{determine_spec_version, metadata_schema, Pact, parse_meta_data, ReadWritePact, verify_metadata};
 use crate::PACT_RUST_VERSION;
+use crate::plugins::PluginData;
 use crate::sync_interaction::RequestResponseInteraction;
 use crate::v4::pact::V4Pact;
 use crate::verify_json::{json_type_of, PactFileVerificationResult, PactJsonVerifier, ResultLevel};
@@ -47,8 +49,8 @@ impl Pact for RequestResponsePact {
     self.provider.clone()
   }
 
-  fn interactions(&self) -> Vec<&dyn Interaction> {
-    self.interactions.iter().map(|i| i as &dyn Interaction).collect()
+  fn interactions(&self) -> Vec<Box<dyn Interaction + Send + Sync>> {
+    self.interactions.iter().map(|i| i.boxed()).collect()
   }
 
   fn metadata(&self) -> BTreeMap<String, BTreeMap<String, String>> {
@@ -86,7 +88,8 @@ impl Pact for RequestResponsePact {
       consumer: self.consumer.clone(),
       provider: self.provider.clone(),
       interactions,
-      metadata: self.metadata.iter().map(|(k, v)| (k.clone(), json!(v))).collect()
+      metadata: self.metadata.iter().map(|(k, v)| (k.clone(), json!(v))).collect(),
+      .. V4Pact::default()
     })
   }
 
@@ -94,11 +97,11 @@ impl Pact for RequestResponsePact {
     self.specification_version.clone()
   }
 
-  fn boxed(&self) -> Box<dyn Pact + Send> {
+  fn boxed(&self) -> Box<dyn Pact + Send + Sync> {
     Box::new(self.clone())
   }
 
-  fn arced(&self) -> Arc<dyn Pact + Send> {
+  fn arced(&self) -> Arc<dyn Pact + Send + Sync> {
     Arc::new(self.clone())
   }
 
@@ -115,6 +118,37 @@ impl Pact for RequestResponsePact {
       }
     }
   }
+
+  fn requires_plugins(&self) -> bool {
+    false
+  }
+
+  fn plugin_data(&self) -> Vec<PluginData> {
+    Vec::default()
+  }
+
+  fn is_v4(&self) -> bool {
+    false
+  }
+
+  fn add_plugin(
+    &mut self,
+    _name: &str,
+    _version: &str,
+    _plugin_data: Option<HashMap<String, Value>>
+  ) -> anyhow::Result<()> {
+    Err(anyhow!("Plugins can only be used with V4 format pacts"))
+  }
+
+  fn add_md_version(&mut self, key: &str, version: &str) {
+    if let Some(md) = self.metadata.get_mut("pactRust") {
+      md.insert(key.to_string(), version.to_string());
+    } else {
+      self.metadata.insert("pactRust".to_string(), btreemap! {
+        key.to_string() => version.to_string()
+      });
+    }
+  }
 }
 
 impl RequestResponsePact {
@@ -163,7 +197,13 @@ impl RequestResponsePact {
       .collect();
 
     md_map.insert("pactSpecification".to_string(), json!({"version" : pact_spec.version_str()}));
-    md_map.insert("pactRust".to_string(), json!({"version" : PACT_RUST_VERSION.unwrap_or("unknown").to_string()}));
+
+    let version_entry = md_map.entry("pactRust".to_string())
+      .or_insert(Value::Object(Map::default()));
+    if let Value::Object(map) = version_entry {
+      map.insert("models".to_string(), Value::String(PACT_RUST_VERSION.unwrap_or("unknown").to_string()));
+    }
+
     md_map
   }
 
@@ -188,8 +228,8 @@ impl RequestResponsePact {
   /// Returns the default metadata
   pub fn default_metadata() -> BTreeMap<String, BTreeMap<String, String>> {
     btreemap!{
-      "pact-specification".to_string() => btreemap!{ "version".to_string() => PactSpecification::V3.version_str() },
-      "pact-rust".to_string() => btreemap!{ "version".to_string() => PACT_RUST_VERSION.unwrap_or("unknown").to_string() }
+      "pactSpecification".to_string() => btreemap!{ "version".to_string() => PactSpecification::V3.version_str() },
+      "pactRust".to_string() => btreemap!{ "models".to_string() => PACT_RUST_VERSION.unwrap_or("unknown").to_string() }
     }
   }
 
@@ -231,10 +271,10 @@ impl ReadWritePact for RequestResponsePact {
     })
   }
 
-  fn merge(&self, pact: &dyn Pact) -> anyhow::Result<Box<dyn Pact>> {
+  fn merge(&self, pact: &dyn Pact) -> anyhow::Result<Box<dyn Pact + Send + Sync>> {
     if self.consumer.name == pact.consumer().name && self.provider.name == pact.provider().name {
-      let conflicts = iproduct!(self.interactions.clone(), pact.interactions().clone())
-        .map(|i| i.0.conflicts_with(i.1))
+      let conflicts = CartesianProductIterator::new(&self.interactions, &pact.interactions())
+        .map(|(i1, i2)| i1.conflicts_with(i2.as_ref()))
         .filter(|conflicts| !conflicts.is_empty())
         .collect::<Vec<Vec<PactConflict>>>();
       let num_conflicts = conflicts.len();
@@ -355,8 +395,3 @@ impl PactJsonVerifier for RequestResponsePact {
     results
   }
 }
-
-#[cfg(test)]
-mod tests {
-
-}
