@@ -3,18 +3,78 @@
 use std::collections::HashMap;
 use std::fmt::{Display, Formatter};
 use std::fmt;
+use std::str::FromStr;
 
 use anyhow::anyhow;
 use log::warn;
-use serde_json::Value;
+use serde_json::{json, Value};
 
 use crate::interaction::Interaction;
 use crate::json_utils::json_to_string;
 use crate::v4::async_message::AsynchronousMessage;
-use crate::v4::sync_message::SynchronousMessages;
+use crate::v4::sync_message::SynchronousMessage;
 use crate::v4::synch_http::SynchronousHttp;
 use crate::v4::V4InteractionType;
 
+/// Markup added to an interaction by a plugin
+#[derive(Debug, Clone, Default, Eq, PartialEq)]
+pub struct InteractionMarkup {
+  /// Markup contents
+  pub markup: String,
+  /// Type of markup (CommonMark or HTML)
+  pub markup_type: String
+}
+
+impl InteractionMarkup {
+  /// Load from a JSON value
+  pub fn from_json(json: &Value) -> InteractionMarkup {
+    match json {
+      Value::Object(values) => InteractionMarkup {
+        markup: values.get("markup").map(|val| json_to_string(val)).unwrap_or_default(),
+        markup_type: values.get("markupType").map(|val| json_to_string(val)).unwrap_or_default()
+      },
+      _ => {
+        warn!("{:?} is not a valid value for InteractionMarkup", json);
+        InteractionMarkup::default()
+      }
+    }
+  }
+
+  /// If this markup is empty
+  pub fn is_empty(&self) -> bool {
+    self.markup.is_empty()
+  }
+
+  /// Convert this markup to JSON form
+  pub fn to_json(&self) -> Value {
+    json!({
+      "markup": self.markup,
+      "markupType": self.markup_type
+    })
+  }
+
+  /// Merges this markup with the other
+  pub fn merge(&self, other: InteractionMarkup) -> InteractionMarkup {
+    if self.is_empty() {
+      other
+    } else if other.is_empty() {
+      self.clone()
+    } else {
+      if self.markup_type != other.markup_type {
+        warn!("Merging different markup types: {} and {}", self.markup_type, other.markup_type);
+      }
+      let mut buffer = String::new();
+      buffer.push_str(self.markup.as_str());
+      buffer.push('\n');
+      buffer.push_str(other.markup.as_str());
+      InteractionMarkup {
+        markup: buffer,
+        markup_type: self.markup_type.clone()
+      }
+    }
+  }
+}
+
 /// V4 Interaction trait
 pub trait V4Interaction: Interaction + Send + Sync {
   /// Convert the interaction to a JSON Value
@@ -37,6 +97,18 @@ pub trait V4Interaction: Interaction + Send + Sync {
 
   /// Type of this V4 interaction
   fn v4_type(&self) -> V4InteractionType;
+
+  /// Any configuration added to the interaction from a plugin
+  fn plugin_config(&self) -> HashMap<String, HashMap<String, Value>>;
+
+  /// Any configuration added to the interaction from a plugin
+  fn plugin_config_mut(&mut self) -> &mut HashMap<String, HashMap<String, Value>>;
+
+  /// Markup added to the interaction to render in UIs
+  fn interaction_markup(&self) -> InteractionMarkup;
+
+  /// Markup added to the interaction to render in UIs
+  fn interaction_markup_mut(&mut self) -> &mut InteractionMarkup;
 }
 
 impl Display for dyn V4Interaction {
@@ -96,27 +168,25 @@ impl PartialEq for Box<dyn V4Interaction> {
 /// Load V4 format interactions from JSON struct
 pub fn interactions_from_json(json: &Value, source: &str) -> Vec<Box<dyn V4Interaction>> {
   match json.get("interactions") {
-    Some(v) => match *v {
-      Value::Array(ref array) => array.iter().enumerate().map(|(index, ijson)| {
+    Some(Value::Array(ref array)) => {
+      array.iter().enumerate().map(|(index, ijson)| {
         interaction_from_json(source, index, ijson).ok()
-      }).filter(|i| i.is_some())
-        .map(|i| i.unwrap())
-        .collect(),
-      _ => vec![]
+      }).flatten()
+        .collect()
     },
-    None => vec![]
+    _ => vec![]
   }
 }
 
 /// Create an interaction from a JSON struct
 pub fn interaction_from_json(source: &str, index: usize, ijson: &Value) -> anyhow::Result<Box<dyn V4Interaction>> {
   match ijson.get("type") {
-    Some(i_type) => match V4InteractionType::from_str(json_to_string(i_type).as_str()) {
+    Some(i_type) => match FromStr::from_str(json_to_string(i_type).as_str()) {
       Ok(i_type) => {
         match i_type {
           V4InteractionType::Synchronous_HTTP => SynchronousHttp::from_json(ijson, index).map(|i| i.boxed_v4()),
           V4InteractionType::Asynchronous_Messages => AsynchronousMessage::from_json(ijson, index).map(|i| i.boxed_v4()),
-          V4InteractionType::Synchronous_Messages => SynchronousMessages::from_json(ijson, index).map(|i| i.boxed_v4())
+          V4InteractionType::Synchronous_Messages => SynchronousMessage::from_json(ijson, index).map(|i| i.boxed_v4())
         }
       },
       Err(_) => {
@@ -131,6 +201,30 @@ pub fn interaction_from_json(source: &str, index: usize, ijson: &Value) -> anyho
   }
 }
 
+pub(crate) fn parse_plugin_config(json: &Value) -> HashMap<String, HashMap<String, Value>> {
+  if let Some(config) = json.get("pluginConfiguration") {
+    match config {
+      Value::Object(map) => map.iter()
+        .map(|(k, v)| {
+          let inner_config = match v {
+            Value::Object(o) => o.iter().map(|(k, v)| (k.clone(), v.clone())).collect(),
+            _ => {
+              warn!("Plugin {} configuration is not correctly formatted, ignoring it", k);
+              Default::default()
+            }
+          };
+          (k.clone(), inner_config)
+        }).collect(),
+      _ => {
+        warn!("Plugin configuration is not correctly formatted, ignoring it");
+        Default::default()
+      }
+    }
+  } else {
+    Default::default()
+  }
+}
+
 #[cfg(test)]
 mod tests {
   use expectest::prelude::*;
